import math;
import graph3;

// Routine modife de graph3.asy
picture surface(triple[][] f, bool[][] cond={}, bool outward=false,
               pen surfacepen[][]={{lightgray}}, pen meshpen=nullpen,
               light light=currentlight, projection P=currentprojection)
{
 picture pic;
 surfacepen.cyclic(true);
 if(!rectangular(f)) abort("matrix is not rectangular");

 // Draw a mesh in the absence of lighting (override with meshpen=invisible).
 if(light.source == O && meshpen == nullpen) meshpen=currentpen;

 int nx=f.length-1;
 int ny=nx > 0 ? f[0].length-1 : 0;

 // calculate colors at each point
 pen color(int i, int j) {
   triple dfx,dfy;
   if(i == 0) dfx=f[1][j]-f[0][j];
   else if(i == nx) dfx=f[nx][j]-f[nx-1][j];
   else dfx=0.5(f[i+1][j]-f[i-1][j]);
   if(j == 0) dfy=f[i][1]-f[i][0];
   else if(j == ny) dfy=f[i][ny]-f[i][ny-1];
   else dfy=0.5(f[i][j+1]-f[i][j-1]);
   triple v=cross(dfx,dfy);
   if(!outward)
     v *= sgn(dot(v,P.camera-P.target));
   return light.intensity(v)*surfacepen[i][j];
 }

 int[] edges={0,0,0,2};

 void drawcell(int i, int j) {
   path g=project(f[i][j],P)--project(f[i][j+1],P)--project(f[i+1][j+1],P)--
     project(f[i+1][j],P)--cycle;
   if(surfacepen[i][j] != nullpen) {
     if(light.source == O)
       fill(pic,g,surfacepen[i][j]);
     else {
       pen[] pcell={color(i,j),color(i,j+1),color(i+1,j+1),color(i+1,j)};
       gouraudshade(pic,g,pcell,edges);
     }
   }
   if(meshpen != nullpen) draw(pic,g,meshpen);
 }

 bool all=cond.length == 0;

 if(surfacepen.length == 0) {
   for(int i=0; i < nx; ++i)
     for(int j=0; j < ny; ++j)
       if(all || cond[i][j])
         drawcell(i,j);
 } else {
   // Sort cells by distance from camera
   triple camera=P.camera;
   if(P.infinity)
     camera *= max(abs(minbound(f)),abs(maxbound(f)));

   real[][] depth;
   for(int i=0; i < nx; ++i) {
     for(int j=0; j < ny; ++j) {
       if(all || cond[i][j]) {
         real d=abs(camera-0.25(f[i][j]+f[i][j+1]+f[i+1][j]+f[i+1][j+1]));
         depth.push(new real[] {d,i,j});
       }
     }
   }

   depth=sort(depth);

   // Draw from farthest to nearest
   while(depth.length > 0) {
     real[] a=depth.pop();
     drawcell(round(a[1]),round(a[2]));
   }
 }

 return pic;
}

// Routine modifie de graph3.asy
picture sphericalsurface(real r(real phi, real theta), pair a, pair b,
                        int nu=nmesh, int nv=nu,
                        bool cond(real phi, real theta)=null, bool outward=false,
                        pen surfacepen(real phi, real theta)=null, pen meshpen=nullpen,
                        light light=currentlight, projection P=currentprojection)
{
 if(surfacepen == null)
   surfacepen=new pen(real t, real p){return lightgray;};
 if(nu <= 0 || nv <= 0) return new picture;
 triple[][] v=new triple[nu+1][nv+1];
 bool[][] active;
 pen [][] surfacepens=new pen[nu+1][nv+1];

 bool all=cond == null;
 if(!all) active=new bool[nu+1][nv+1];
 triple P(pair z)
 {
     real rt=r(z.x,z.y);
     return (rt*cos(z.x)*sin(z.y),rt*sin(z.x)*sin(z.y),rt*cos(z.y));
 }
 real du=1/nu;
 real dv=1/nv;
 pair Idv=(0,dv);
 pair dz=(du,dv);
 for(int i=0; i <= nu; ++i) {
   real x=interp(a.x,b.x,i*du);
   for(int j=0; j <= nv; ++j) {
     pair z=(x,interp(a.y,b.y,j*dv));
     v[i][j]=P(z);
     surfacepens[i][j]=surfacepen(z.x,z.y);
     if(!all) active[i][j]=cond(z.x,z.y) || cond(z.x+du,z.y) ||
                cond(z.x,z.y+Idv.y) || cond(z.x+dz.x,z.y+dz.y);
   }
 }
 return surface(v,active,outward,surfacepens,meshpen,light,P);
}

typedef real fct(real);
typedef pair zfct2(real,real);
typedef real fct2(real,real);

real binomial(real n, real k)
{
 return gamma(n+1)/(gamma(n-k+1)*gamma(k+1));
}

int factorial(int n) {
 int i=1;
 for (int k=1; k <= n; ++k) {
   i *= k;
 }
 return i;
}

real[] pdiff(real[] p)
{ // p(x)=p[0]+p[1]x+...p[n]x^n
 // retourne la drive de p
 real[] dif;
 for (int i : p.keys) {
   if(i != 0) dif.push(i*p[i]);
 }
 return dif;
}

real[] pdiff(real[] p, int n)
{ // p(x)=p[0]+p[1]x+...p[n]x^n
 // drive n-ime de p
 real[] dif={0};
 if(n >= p.length) return dif;
 dif=p;
 for (int i=0; i < n; ++i)
   dif=pdiff(dif);
 return dif;
}

fct operator *(real y, fct f)
{
 return new real(real x){return y*f(x);};
}


fct fct(real[] p)
{ // convertit le tableau des coefs du poly p en fonction polynme
 return new real(real x){
   real y=0;
   for (int i : p.keys) {
     y += p[i]*x^i;
   }
   return y;
 };
}

fct P(int l, int m)
{ // Polynme de Legendre associ
 // http://mathworld.wolfram.com/LegendrePolynomial.html
 if(m < 0) return (-1)^(-m)*factorial(l+m)/factorial(l-m)*P(l,-m);
 real[] xl2;
 for (int k=0; k <= l; ++k) {
   xl2.push((-1)^(l-k)*binomial(l,k));
   if(k != l) xl2.push(0);
 }
 fct dxl2=fct(pdiff(xl2,l+m));
 return new real(real x){
   return (-1)^m/(2^l*factorial(l))*(1-x^2)^(m/2)*dxl2(x);
 };
}

zfct2 Y(int l, int m)
{// http://fr.wikipedia.org/wiki/Harmonique_sph%C3%A9rique#Expression_des_harmoniques_sph.C3.A9riques_normalis.C3.A9es
 return new pair(real phi, real theta) {
   return sqrt((2*l+1)*factorial(l-abs(m))/(4*pi*factorial(l+abs(m))))*P(l,m)(cos(theta))*expi(m*phi);
 };
}
